%This is a simple smiulator for an underwater snake-like robot. The code is
%not optimized for speed and there may still be bugs. Please report back to
%me if you find anything fishy :)
%
%Henrik

clc;
clear
close all

C_1 = eye(29,101);
C_2 = [zeros(72,29) eye(72,72)];

% C_1 = eye(58,137);
% C_2 = [zeros(79,58) eye(79,79)];

%% Define Physical Parameters
pp.denw = 1000;         %density of water
pp.grav = 9.81;         %gravitational acceleration
pp.velc = [0 0 0]';     %constant irrotational current velocity, inertial frame
pp.gdir = [0 0 -1]';    %direction of gravity in inertial frame (pointing towards ground, unit vector

%% Define Snake Parameters
% snake.link_vec  = [8 8 8 8 8];         %The links the snake consists of, from head to tail
% snake.joint_vec = [3 3 3 3];           %The type of joints, from head to tail
snake.link_vec  = [1 2 3 2 4 2 5 2 6];         
snake.joint_vec = [3 2 3 2 3 2 3 2];          
snake.n         = length(snake.link_vec);

C_a_C   =   1; % Added mass coefficient for the cross section, 1 is the theoretical inviscid result
C_d_1   =  .2; % nonlinear drag coefficient in surge
C_d_4   =  .1; % nonlinear drag coefficient in the roll direction for the cross-section
C_d_C   =  .5; % nonlinear crossflow drag coefficient
C_d_L   =  .1; % Linear cross-sectional drag coefficient
alpha   =  .2; % Added mass ratio in surge/heave for a link
beta    =  .1; % Linear drag parameter in surge
gamma   =  .1; % Linear drag parameter in roll

masscoeff = [C_a_C, alpha];
dragcoeff = [C_d_1, C_d_4, C_d_C, C_d_L, beta, gamma];
clear C_a_C C_d_C C_d_1 C_d_4 C_d_L alpha beta gamma

%% Define Link Properties
link = define_link_properties_eely(pp, masscoeff, dragcoeff);

%% Define Joint Properties
joint = define_joint_properties();
                  
%% Define Simulation Parameters
Tmax          = 100;
deltaT_interp = 1;                     %Time step size for the interpolated solution
deltaT_plot   = .1;                      %Time step size for the plotted solution

y_0             = zeros(29,1); %Initial conditions
y_0(3)          = -20;
y_0(4)          = 1;                     %"zero" quaternion
%  y_0(8:snake.n+6) = -[pi/6 0 pi/6 0 pi/6 0 pi/6 0]'; 

% y_0(8:snake.n+6) = [pi/6 0 pi/6 0 pi/6 0 pi/6 0]'; % initialize with
%U-shape
% y_0(3) = 25;
                  
%% Build Snake
snake = build_snake(snake, link, joint, pp);
% Simulink.data.evalinGlobal('USMsim','snake = build_snake(snake, link, joint, pp);')

%% Simulate
disp('Simulation starting:')
disp('...')
% [T, simout]   = ode45(@(t,x)dyn(t,x,snake,pp,0),[0 Tmax], y_0);
options = simset('SrcWorkspace','current');
sim('opspace.slx',Tmax,options)
disp('Simulation done!')

%% Post Process/Show Movie
% this code shows the correct result if all links have the same radius!
% disp('Postprocessing data:')
% disp('...')
% Tinterp      = 0 : deltaT_interp : Tmax;
% simout_int   = interp1(T,simout,Tinterp);
% TF = ismissing(simout_int);
% k = find(TF,1);
% if isempty(k), k = length(Tinterp); end
% simout_int = simout_int(1:k-1,:);
% [X, Y, Z]    = snake2points(simout_int,snake);
% mov          = animatesnake(X,Y,Z,snake);

% Tinterp      = 0 : deltaT_plot : Tmax;
% simout_int   = interp1(T,simout,Tinterp);
% TF = ismissing(simout_int);
% k = find(TF,1);
% if isempty(k), k = length(Tinterp); end
% simout_int = simout_int(1:k-1,:);
% simout = dyn(Tinterp(1),simout_int(1,:)',snake,pp,1);
% simout = [simout, zeros(length(simout),k-2)];
% for t = 2:k-1
%     simout(:,t) = dyn(Tinterp(t),simout_int(t,:)',snake,pp,1);
% end
% save('log_moe.mat', 'simout')
% disp('Postprocessing done.')
%%
% [X, Y, Z]    = snake2points(simout.signals.values,snake);
% mov          = animatesnake(X,Y,Z,snake);


figure
hold on;
plot(simout.time, simout.signals.values(:,25:25+7)')
xlabel('$t$ [s]','interpreter','latex');
ylabel('$u$ [Nm]','interpreter','latex')
title('Control torque')
grid on

figure
plot(simout.time, simout.signals.values(:,18:24)')
xlabel('$t$ [s]','interpreter','latex');
ylabel('$u$ [N]','interpreter','latex')
title('Thruster force') 
grid on






%% Custom plotting
% figure
% hold on
% plot(simout.time, simout.signals.values(:,2))
% plot(simout.time, simout(33,:)', '--')
% title('Surge velocity')     

% figure
% hold on;
% plot(simout.time, (simout.signals.values(:,4:9) - simout.signals.values(:,33:38)))
% title('Velocity errors')


figure
th = 0:pi/50:2*pi;
xunit = 1.5 * cos(th) + 2;
yunit = 1.5 * sin(th) + 2;
hold on
plot(simout.signals.values(:,2)', simout.signals.values(:,1)')
plot(simout.signals.values(:,34)', simout.signals.values(:,33)')
plot(xunit, yunit);
legend({'$p_b$', '$p_e$'},'Interpreter','latex')
title('North-East plot')

% figure
% hold on
% plot3(simout(2,:)', simout(1,:)', simout(3,:)')
% plot3(simout(48,:)', simout(47,:)', simout(49,:)')
% legend({'$p_b$', '$p_e$'},'Interpreter','latex')
% title('North-East-Down plot')
%%

% figure
% hold on;
% plot(simout.time,180/pi.*simout(10:17,:)')
% plot(simout.time, 180/pi.*simout(39:46,:)', '--')
% title('Joint angles')

figure
hold on;
plot(simout.time, 180/pi.*simout.signals.values(:,48:55)')
% plot(simout.time, 180/pi.*simout.signals.values(:,70:77)', '--')
title('Joint angles')

figure
plot(simout.time, simout.signals.values(:,56))
title('Mode')
%%


figure
hold on;
plot(simout.time, simout.signals.values(:,1)', '-g')
plot(simout.time, simout.signals.values(:,45)','--')
plot(simout.time, simout.signals.values(:,33)', 'r')
plot(simout.time, simout.signals.values(:,39)','--')
legend({'$x_b$', '$x_{b,d}$', '$x_e$', '$x_{e,d}$'},'Interpreter','latex')

figure
hold on;
plot(simout.time, simout.signals.values(:,2)', '-g')
plot(simout.time, simout.signals.values(:,46)', '--')
plot(simout.time, simout.signals.values(:,34)', 'r')
plot(simout.time, simout.signals.values(:,40)', '--')
legend({'$y_b$', '$y_{b,d}$','$y_e$','$y_{e,d}$'},'Interpreter','latex')

figure
hold on;
plot(simout.time, simout.signals.values(:,3)', '-g')
plot(simout.time, simout.signals.values(:,47)', '--')
plot(simout.time, simout.signals.values(:,35)', 'r')
plot(simout.time, simout.signals.values(:,41)', '--')
legend({'$z_b$', '$z_{b,d}$','$z_e$','$z_{e,d}$'},'Interpreter','latex')


figure
hold on;
plot(simout.time, 180/pi.*simout.signals.values(:,36)')
plot(simout.time, 180/pi.*simout.signals.values(:,37)')
plot(simout.time, 180/pi.*simout.signals.values(:,38)')
plot(simout.time, 180/pi.*simout.signals.values(:,42), '--')
plot(simout.time, 180/pi.*simout.signals.values(:,43), '--')
plot(simout.time, 180/pi.*simout.signals.values(:,44), '--')
legend({'$\phi$','$\theta$','$\psi$','$\phi_d$','$\theta_d$', '$\psi_d$'},'interpreter','latex')


figure
hold on
plot(simout.time, simout.signals.values(:,57:70));
title('Reference accelerations')

figure
hold on
plot(simout.time, simout.signals.values(:,71));
title('Distance between end-effector and obstacle')

figure
hold on
plot(simout.time, simout.signals.values(:,72));
title('Change in distance from end-effector to obstacle')



% fig = figure;
% movie(fig,mov,50)

